part of 'bolter_provider.dart';

class BolterBuilderController<T> {
  final BolterInterface bolter;
  late T _value;
  final _notifications = <BolterNotification>[];

  BolterBuilderController(this.bolter, this._value);

  T get value => _value;

  void update(T Function(T value) action) {
    for (final notification in _notifications) {
      bolter.runAndUpdate(
        action: () => _value = action(_value),
        exactNotification: notification,
      );
    }
  }

  void dispose() {
    _notifications.clear();
  }
}

class BolterBuilder<T> extends StatefulWidget {
  final T Function()? getter;
  final Widget Function(BuildContext context, T value) builder;
  final BolterBuilderController<T>? controller;

  const BolterBuilder({
    Key? key,
    required this.getter,
    required this.builder,
  })  : controller = null,
        super(key: key);

  const BolterBuilder.controller({
    Key? key,
    required this.controller,
    required this.builder,
  })  : getter = null,
        super(key: key);

  @override
  _BolterBuilderState<T> createState() => _BolterBuilderState<T>();
}

class _BolterBuilderState<T> extends State<BolterBuilder<T>> {
  late final bolter = context.bolter;

  void _notification() {
    setState(() {});
  }

  @override
  void initState() {
    super.initState();
    widget.controller?._notifications.add(_notification);
    bolter.listen(widget.getter ?? () => widget.controller!._value, _notification);
  }

  @override
  void didUpdateWidget(BolterBuilder<T> oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.getter != widget.getter || oldWidget.controller != widget.controller) {
      bolter.stopListen(_notification);
      bolter.listen(widget.getter ?? () => widget.controller!._value, _notification);
    }
  }

  @override
  Widget build(BuildContext context) => widget.builder(
        context,
        widget.getter != null ? widget.getter!() : widget.controller!._value,
      );

  @override
  void dispose() {
    widget.controller?._notifications.remove(_notification);
    bolter.stopListen(_notification);
    super.dispose();
  }
}
